/*
 *  Libjpeg interface routines.
 *  We support JPEG decompress interface only.
 */
#include <stdio.h>
#include "viewer.h"
#include "../jpeg/jpeglib.h"
#include "../lcdtext/grphcon.h"

#define	POOL_SIZE	(1024L*50L)       /* Hope this is big enough */

/*
 *  WJԂƎgpʂጸ邽߂ɁAJPEG̉摜TCY
 *  PIC_WIDTH X PIC_HEIGHT sNZɐ.
 */
#define	PIC_WIDTH	400
#define	PIC_HEIGHT	300

struct jpeg_error_mgr jerr;
struct jpeg_error_mgr *jpeg_std_error (struct jpeg_error_mgr * err);
void jpeg_fatfs_src (j_decompress_ptr cinfo, FIL * infile);

void LCD_Draw_Line(int orientation, unsigned char *bp, int xoffset, int ypos, int width);

struct jpeg_decompress_struct dcinfo;

static FIL jpeg_file;

char mem_pool[POOL_SIZE];

static char *pool_ptr;
static char *ext_pool_ptr;
extern char ex_ram_pool[];
#define	EXT_POOL_SIZE (256*1024-240*320*2)

void pool_init()
{
    pool_ptr = mem_pool;
    ext_pool_ptr = ex_ram_pool;
}

void pool_stat()
{
#ifdef VERBOSE
    xprintf("%d used.\n", pool_ptr - mem_pool);
#endif
}

char *malloc(int size)
{
    char *mp;

    mp = pool_ptr;
    if (pool_ptr + size > mem_pool + POOL_SIZE) {
      mp = ext_pool_ptr;
      if (ext_pool_ptr + size > ex_ram_pool + EXT_POOL_SIZE) {
#ifdef VERBOSE
        xprintf("No memory!\n");
#endif
        return NULL;
      }
      ext_pool_ptr += size;
    } else {
      pool_ptr += size;
    }
    return mp;
}

void free(char *mp)
{
    /* DO NOTHING */
}

FIL *open_jpeg_file(char *fname)
{
    FRESULT result;

    result = f_open(&jpeg_file, fname, FA_READ);
    if (result != FR_OK) {
        return NULL;
    }
    return &jpeg_file;
}

void close_jpeg_file(FIL *fptr)
{
    f_close(fptr);
}

/* 
 *  JPEG 摜WJ
 *    fname:  JPEG t@C
 *    orientation:  JPEG 摜̕\
 */
int expand_jpeg(char *fname, EXIF *exif)
{
    int denom, scale;
    FIL *fptr;
    JSAMPARRAY buffer;
    int row_stride;
    int xoffset, yoffset, line;
    int hoffset, voffset;
    int picw, pich, maxw, maxh;
    int screenw, screenh;
    int dw;
    int orientation = exif->orientation;

#ifdef DEBUG
xprintf("fname = %s\n", fname);
#endif
    if (orientation < 1 || orientation > 8)
	orientation = 1;
    pool_init();
    fptr = open_jpeg_file(fname);
    if (!fptr)  return 0;

    dcinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&dcinfo);
    jpeg_fatfs_src(&dcinfo, fptr);

    jpeg_read_header(&dcinfo, TRUE);

    /*
     * screenw, screenh: LCDʂ̕A
     * picw, pich:       zXN[̃TCYBLCD̎ʃTCY傫
     * maxw, maxh:       JPEG摜̍őTCY. pcw/pich 8{.
     */
    if (orientation < 5) {	/* Landscape */
        screenw = LCD_WIDTH;
        screenh = LCD_HEIGHT;
	picw = PIC_WIDTH;
	pich = PIC_HEIGHT;
        maxw = PIC_WIDTH * 8;
        maxh = PIC_HEIGHT * 8;
    } else {			/* Portrait */
	screenw = LCD_HEIGHT;
	screenh = LCD_WIDTH;
	picw = PIC_HEIGHT;
	pich = PIC_WIDTH;
        maxw = PIC_HEIGHT * 8;
	maxh = PIC_WIDTH * 8;
    }

    /*
     * {\tgł́Amaxw * maxh 傫ȉ摜͈ȂBG[ƂB
     */
    if ((dcinfo.image_width > maxw) || (dcinfo.image_height > maxh)) {
#ifdef DEBUG
        xprintf("image too big: %d x %d\n", dcinfo.image_width, dcinfo.image_height);
#endif
        /* Image is too big */
        jpeg_destroy_decompress(&dcinfo);
        pool_init();
        f_lseek(fptr, exif->thumb_offset);
    jpeg_create_decompress(&dcinfo);
    jpeg_fatfs_src(&dcinfo, fptr);

    jpeg_read_header(&dcinfo, TRUE);
    orientation = 1;
        screenw = LCD_WIDTH;
        screenh = LCD_HEIGHT;
	picw = PIC_WIDTH;
	pich = PIC_HEIGHT;
        scale = 16;
        goto doscale;
    }
    dcinfo.scale_denom = dcinfo.scale_num = 8;
    denom = 8;

    for (scale = 8; scale > 1; scale--) {
        if (dcinfo.image_width / denom * scale <= picw)
            break;
    }
    while (dcinfo.image_height/denom * scale > pich) {
        scale--;
    }
doscale:
    if (scale > 0) {
        dcinfo.scale_num = scale;
        dcinfo.out_color_space = JCS_RGB;
        dcinfo.dct_method = JDCT_ISLOW;
        jpeg_start_decompress(&dcinfo);
#ifdef DEBUG
xprintf("%d x %d --> %d x %d\n",
  dcinfo.image_width, dcinfo.image_height,
  dcinfo.output_width, dcinfo.output_height);
#endif

        xoffset = yoffset = 0;
	hoffset = voffset = 0;

        /* wiœhԂ */
        while (vb_check() == 0);
	GRPH_Clear(C_BLACK);

        /* 摜Z^Oĕ\ */
        if (dcinfo.output_width <= screenw) {
                xoffset = (screenw - dcinfo.output_width) >> 1;
	} else {
		hoffset = (dcinfo.output_width - screenw) >> 1;
	}
	if (dcinfo.output_height <= screenh) {
                yoffset = (screenh - dcinfo.output_height) >> 1;
        } else {
		voffset = (dcinfo.output_height - screenh) >> 1;
	}
#ifdef DEBUG
xprintf("hoffset = %d, voffset = %d\n", hoffset, voffset);
#endif

        row_stride = dcinfo.output_width * dcinfo.output_components;
        buffer = (*dcinfo.mem->alloc_sarray)((j_common_ptr)&dcinfo, JPOOL_IMAGE, row_stride, 1);
	dw = dcinfo.output_width;
	dw -= (hoffset << 1);
 
        while (dcinfo.output_scanline < dcinfo.output_height) {
	    line = dcinfo.output_scanline;
            jpeg_read_scanlines(&dcinfo, buffer, 1);
	    if (line >= voffset) {
		LCD_Draw_Line(orientation, buffer[0] + hoffset * 3,
		    xoffset, yoffset + line - voffset, dw);
	    }
        }
        pool_stat();

        jpeg_finish_decompress(&dcinfo);
    }
    jpeg_destroy_decompress(&dcinfo);
    close_jpeg_file(fptr);

    return 1;
}

/* 
 *  JPEG TlC摜̐LA\
 *    fname:  JPEGt@C
 *    thumb_offet:  JPEGt@CATlC摜̂ʒuւ̃ItZbg
 *    picno:        \ʒuԍ (0..11)
 *
 * Picno ̒lɉāALCDʏ̎̈ʒuɉ摜\
 *
 *   0: xoffset = 0,   yoffset = 0
 *   1: xoffset = 80,  yoffset = 0
 *   2: xoffset = 160, yoffset = 0
 *   3: xoffset = 240, yoffset = 0
 *   4: xoffset = 0,   yoffset = 60
 *   5: xoffset = 80,  yoffset = 60
 *   6: xoffset = 160, yoffset = 60
 *   7: xoffset = 240, yoffset = 60
 *   8: xoffset = 0,   yoffset = 120
 *   9: xoffset = 80,  yoffset = 120
 *  10: xoffset = 160, yoffset = 120
 *  11: xoffset = 240, yoffset = 120
 */
int expand_thumbnail(char *fname, long thumb_offset, int picno)
{
    int scale;
    int orientation;
    FIL *fptr;
    JSAMPARRAY buffer;
    int row_stride;
    int xoffset, yoffset, line;
    int picw, pich;
    int dw, mod12;
int dh;

#ifdef DEBUG
    xprintf("fname = %s\n", fname);
#endif
    if (picno >= 0) {
        /* \ʒuԍw肳Ă̂ŁA̍W߂ */
        mod12 = picno % 12;
        xoffset = 80 * (mod12 & 3);
        yoffset = 60 * (mod12 / 4);
        picw = 80;
        pich = 60;
    } else {
       /* ŕ\ */
       xoffset = yoffset = 0;
       picw = 160; pich = 120;
    }
    if (thumb_offset == 0) {
        /* TlC摜܂܂ĂȂB̎|\B*/
        GRPH_Box(xoffset, yoffset, xoffset + picw - 1, yoffset + pich - 1, C_BLACK, 1);
	xoffset += picw / 2;
	yoffset += pich / 2;
        GCON_locate(xoffset /sc_col_width - 1, yoffset / sc_row_height);
	t_puts("No");
        GCON_locate(xoffset /sc_col_width - 3, yoffset / sc_row_height + 1);
        t_puts("Thumb");
        return 0;
    }

    pool_init();
    fptr = open_jpeg_file(fname);
    if (!fptr)  return 0;

    /* For thumbnail, seek to its start position */
    f_lseek(fptr, thumb_offset);
	
    dcinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&dcinfo);
    jpeg_fatfs_src(&dcinfo, fptr);

    jpeg_read_header(&dcinfo, TRUE);

    /*
     * Thumbnail is always displayed in landscape mode.
     */
    orientation = 1;
    dw = dcinfo.image_width;
    dh = dcinfo.image_height;

    /* Verify picture size */
    if ((dw != 160) || (dh != 120)) {
        if (dw <= 160 && dh <= 120) {
            orientation = 1;
        } else if (dw <= 120 && dh <= 160) {
            orientation = 6;
	} else {
#ifdef DEBUG
        xprintf("Invalid thumbnail size: %d x %d\n", dcinfo.image_width, dcinfo.image_height);
#endif
        /* Image is too big */
        jpeg_destroy_decompress(&dcinfo);
        close_jpeg_file(fptr);
        return 0;
	}
    }
    dcinfo.scale_denom = dcinfo.scale_num = 8;

    if (picno >= 0) {
	/*
	 * JPEG thumbnail picture is 160 x 120 dots.
	 * We'll scale it to half size (80 x 60)
         */
	scale = 4;	/* scale 4/8 */
    } else {
	scale = 8;	/* Use original 160 x 120 dots */
    }

    dcinfo.scale_num = scale;
    dcinfo.out_color_space = JCS_RGB;
    dcinfo.dct_method = JDCT_ISLOW;
    jpeg_calc_output_dimensions(&dcinfo);
#ifdef DEBUG
    xprintf("%d x %d --> %d x %d\n",
        dcinfo.image_width, dcinfo.image_height,
        dcinfo.output_width, dcinfo.output_height);
#endif

    jpeg_start_decompress(&dcinfo);

    row_stride = dcinfo.output_width * dcinfo.output_components;
    buffer = (*dcinfo.mem->alloc_sarray)((j_common_ptr)&dcinfo, JPOOL_IMAGE, row_stride, 1);
    dw = dcinfo.output_width;
 
    while (dcinfo.output_scanline < dcinfo.output_height) {
	line = dcinfo.output_scanline;
        jpeg_read_scanlines(&dcinfo, buffer, 1);
	LCD_Draw_Line(orientation, buffer[0], xoffset, yoffset + line, dw);
    }
#ifdef DEBUG
    pool_stat();
#endif

    jpeg_finish_decompress(&dcinfo);
    jpeg_destroy_decompress(&dcinfo);
    close_jpeg_file(fptr);

    return 1;
}
