/*
 * RX62N data flash handling routines
 */
#include "iodefine.h"
#include "../lcdtext/touch.h"
#include "../lcdtext/lcdset.h"
#include "../lcdtext/graphics.h"
#include "../lcdtext/grphcon.h"

#pragma section DFLASH
volatile unsigned char df_block0[1024*2];
#pragma section

/* We use first one block */
#define	DFTOP_ADDR	df_block0	/* f[^}bg̐擪AhX */

#define	RESET_KEY	0xcc
#define	DBRE0_KEY	0x2d
#define	DBWE0_KEY	0x1e
#define	FE_KEY	0xaa

#define	DF_MAGIC0	0x5450
#define	DF_MAGIC1	0x4344

#define	FCU_NORMAL_MODE	0xff
#define	FCU_BLOCK_ERASE	0x20
#define	FCU_PROGRAM	0xe8
#define	FCU_CLOCK_SPEED	0xe9
#define	FCU_COM_END	0xd0

void df_unprotect()
{
   FLASH.FWEPROR.BYTE = 0x01;
   /* Allow read/write on the first 2KB blok */
   FLASH.DFLWE0.WORD = (DBWE0_KEY << 8) | 1;
   FLASH.DFLRE0.WORD = (DBRE0_KEY << 8) | 1;
}

int df_check(TP_CALIBRATION *tpc)
{
    short *wp = (short *)DFTOP_ADDR;

    df_unprotect();	/* f[^tbVւ̃ANZX */

    if (wp[0] == DF_MAGIC0 && wp[1] == DF_MAGIC1) {
        tpc->x_min = wp[2];
        tpc->x_max = wp[3];
        tpc->y_min = wp[4];
        tpc->y_max = wp[5];
        return 1;
    }
    return 0;
}


void df_write(TP_CALIBRATION *tpc)
{
    volatile char *bp;
    volatile short *wp;
    int nw;
    volatile unsigned char stat;

    df_unprotect();

    /* f[^tbV P/E m[}[hɈڍs */
    FLASH.FENTRYR.WORD = (FE_KEY << 8) | 0x80;


    /* ӃNbNʒmR}h𔭍s */

    FLASH.PCKAR.WORD = PCLK;

    delay_ms(10);

    bp = (char *)DFTOP_ADDR;

    wp = (short *)DFTOP_ADDR + 0;

    *bp = FCU_CLOCK_SPEED;

    *bp = 0x03;

    *wp = 0x0f0f;
    *wp = 0x0f0f;

    *wp = 0x0f0f;

    *bp = FCU_COM_END;

	delay_ms(200);
	if (!(FLASH.FSTATR0.BYTE & 0x80)) {
		stat = FLASH.FSTATR0.BYTE;
		 t_printf("@0 %x, %x", stat, FLASH.FASTAT.BYTE);
	}
    /* FRDYrbgmF */
    while (!(FLASH.FSTATR0.BYTE & 0x80)) ;
	
	stat = FLASH.FSTATR0.BYTE;


    /* ubNC[YR}h */
    *bp = FCU_BLOCK_ERASE;
    *bp = FCU_COM_END;
	delay_ms(400);
    /* FRDYrbgmF */
    while (!(FLASH.FSTATR0.BYTE & 0x80)) ;


    /* f[^tbV̈̃AhXE8hoCg */
    *bp = FCU_PROGRAM;
    /* f[^tbV̈̃AhXɏ݃[h(N)oCg */
    *bp = 0x40;		/* 128oCg */


    /* ݐ̐擪AhXɏ݃f[^[h */
    *wp = DF_MAGIC0;	/* Magic numbeȑ */
    *wp = DF_MAGIC1;

    *wp = tpc->x_min;	/* Lu[Vf[^̏ */
    *wp = tpc->x_max;
    *wp = tpc->y_min;
    *wp = tpc->y_max;

    for (nw = 6; nw < 64; nw++)
      *wp = 0xffff;	/* c̗̈ 0xffff Ŗ߂ */

    /* f[^tbV̈̃AhXD0hoCg */
    *bp = FCU_COM_END;
	delay_ms(200);

    /* FRDYrbgmF */
    while (!(FLASH.FSTATR0.BYTE & 0x80)) ;
    /* f[^tbV P/E m[}[hɈڍs */

    /* f[^tbV [h[hɈڍs */
    FLASH.FENTRYR.WORD = (FE_KEY << 8) | 0x00;
}
