#include "iodefine.h"
#include "event.h"
#include "touch.h"
#include "vect.h"

#define	EVQLEN	30

volatile evinfo *ev_top;
volatile evinfo *ev_tail;
static struct s_timer tick_timer;

static evinfo qdata[EVQLEN];

static void tick_timer_increment()
{
    tick_timer.counter++;
}

int get_time_tick()
{
    return tick_timer.counter;
}

void add_tick_timer()
{
    tick_timer.tfunc = tick_timer_increment;
    add_timer(&tick_timer);
}

void ev_init()
{
    ev_tail = ev_top = &qdata[0];
}

void ev_add(int event, int xpos, int ypos)
{
    ev_tail->ev_type = event;
    ev_tail->ev_time = get_time_tick();
    ev_tail->xpos = xpos;
    ev_tail->ypos = ypos;
    ev_tail++;
    if (ev_tail == &qdata[EVQLEN]) ev_tail = &qdata[0];
}

evinfo *ev_get(int do_wait)
{
    volatile evinfo *evp;
    volatile int us;

    while (1) {
        if (ev_top != ev_tail)
            break;
        if (!do_wait) return (evinfo*)0;
    }
    evp = ev_top++;
    if (ev_top == &qdata[EVQLEN]) ev_top = &qdata[0];
    return (evinfo *)evp;
}

void ev_flush()
{
    while (ev_get(0)) {
        delay_ms(2);
    }
}

void ev_wait(int wevent)
{
    evinfo *evp;

    do {
        evp = ev_get(1);
    } while (evp->ev_type != wevent);
}

#define	RIGHTKEY_MASK	(1<<0)
#define	LEFTKEY_MASK	(1<<1)
#define	DOWNKEY_MASK	(1<<2)
#define	UPKEY_MASK	(1<<3)
#define	ASELKEY_MASK	(1<<4)
#define	BSELKEY_MASK	(1<<5)

void key_timerproc()
{
    static short prev_scan, key_state;
    short work, change;

    work = ((PORT4.PORT.BYTE >> 4) & 0x0f) | ((PORT6.PORT.BYTE >> 2) & 0x30);
    if (work == prev_scan) {
        change = key_state;
        key_state = work;    /* update key status */
        change = change ^ work;
        if (change) {
            if ((change & RIGHTKEY_MASK) && !(work & RIGHTKEY_MASK)) {
                ev_add(EV_RIGHTKEY_DOWN, 0, 0);
            }
            if ((change & LEFTKEY_MASK) && !(work & LEFTKEY_MASK)) {
                ev_add(EV_LEFTKEY_DOWN, 0, 0);
            }
            if ((change & DOWNKEY_MASK) && !(work & DOWNKEY_MASK)) {
                ev_add(EV_DOWNKEY_DOWN, 0, 0);
            }
            if ((change & UPKEY_MASK) && !(work & UPKEY_MASK)) {
                ev_add(EV_UPKEY_DOWN, 0, 0);
            }
            if ((change & ASELKEY_MASK) && !(work & ASELKEY_MASK)) {
                ev_add(EV_ASELKEY_DOWN, 0, 0);
            }
            if ((change & BSELKEY_MASK) && !(work & BSELKEY_MASK)) {
                ev_add(EV_BSELKEY_DOWN, 0, 0);
            }
        }
    }
    prev_scan = work;
}

// CMTU0_CMT1
void Excep_CMTU0_CMT1(void){
	key_timerproc();
}
