/***********************************************************************/
/*                                                                     */
/*  FILE        :touch.c                                               */
/*  DESCRIPTION :check touch and position                              */
/*  CPU TYPE    :RX62N                                                 */
/*                                                                     */
/***********************************************************************/
#include <machine.h>
#include <string.h>
#include "vect.h"
#include "iodefine.h"
#include "lcdset.h"
#include "touch.h"
#include <stdio.h>
#include "event.h"
#include "../lcdtext/graphics.h"
#include "../lcdtext/grphcon.h"

static volatile unsigned short global_inX;
static volatile unsigned short global_inY;
volatile unsigned short global_touch;

static volatile int delay_Timer;

static struct s_timer *timer_list;

static int pen_down_x, pen_down_y;

TP_CALIBRATION tp_calib_data;

void add_timer(struct s_timer *ptimer)
{
    ptimer->next = timer_list;
    timer_list = ptimer;
}

void delay_ms (					/* msPʂ̃fBC */
	int ms
)
{
    for (delay_Timer = 0; delay_Timer < ms; ) ;
}

// CMTU0_CMT0
//^b`pl̏擾@1msœ삷銄荞
void Excep_CMTU0_CMT0(void)
{ 
	static int t_state;
        struct s_timer *tlist;

	delay_Timer++;
        for (tlist = timer_list; tlist; tlist = tlist->next)
            (*tlist->tfunc)();

	switch(t_state){					//oNo.mF
	case 0:								//0F^b`ꂽo
		PORTC.DR.BYTE = T_sense_DR;
		PORTC.DDR.BYTE = T_sense_DDR;
		t_state++;
		break;
	case 1:								//1F^b`Ă2
		if(PORT4.PORT.BIT.B0 == ON){	
			t_state++;
		} else {						//@^b`ĂȂΌo0
			PORTC.DR.BYTE = T_idle_DR;
			PORTC.DDR.BYTE = T_idle_DDR;				
			if (global_touch) ev_add(EV_PENUP, pen_down_x, pen_down_y);
			global_touch = 0;
			t_state = 0;
		}
		break;
	case 2:								//`^Ôߑ҂
	case 3:
		t_state++;
		break;
	case 4:								//4F^b`Ɣ
		if(PORT4.PORT.BIT.B0 == ON){	
			PORTC.DR.BYTE = T_posY_DR;	//yʒuopd
			PORTC.DDR.BYTE = T_posY_DDR;
			t_state++;
		} else {						//@^b`ȂȂ猟o0
			PORTC.DR.BYTE = T_idle_DR;
			PORTC.DDR.BYTE = T_idle_DDR;				
			if (global_touch) ev_add(EV_PENUP, pen_down_x, pen_down_y);
			global_touch = 0;
			t_state = 0;
		}
		break;
	case 5:								//5FyôA-DRo[^N
		S12AD.ADANS.WORD = 1;			//AN0 enable
		S12AD.ADCSR.BIT.ADST = 1;		//strat
		t_state++;
		break;
	case 6:								//6F^b`opd
		PORTC.DR.BYTE = T_sense_DR;
		PORTC.DDR.BYTE = T_sense_DDR;
		t_state++;
		break;
	case 7:								//7F^b`Ɣ
		if(PORT4.PORT.BIT.B0 == ON){	//
			PORTC.DR.BYTE = T_posX_DR;	//xʒuopd
			PORTC.DDR.BYTE = T_posX_DDR;
			t_state++;
		} else {						//@^b`ȂȂ猟o0
			PORTC.DR.BYTE = T_idle_DR;
			PORTC.DDR.BYTE = T_idle_DDR;				
			if (global_touch) ev_add(EV_PENUP, pen_down_x, pen_down_y);
			global_touch = 0;
			t_state = 0;
		}
		break;
	case 8:								//8FxôA-DRo[^N
		S12AD.ADANS.WORD = 2;			//AN1 enable
		S12AD.ADCSR.BIT.ADST = 1;		//strat
		t_state++;
		break;
	case 9:								//9F^b`opd
		PORTC.DR.BYTE = T_sense_DR;	
		PORTC.DDR.BYTE = T_sense_DDR;
		t_state++;
		break;
	case 10:							//`^Ô߂̑҂
	case 11:
		t_state++;
		break;
	case 12:							//12F^b`mF
		if(PORT4.PORT.BIT.B0 == ON){	//A-Dϊʂ荞
			int x, y;
                        TP_CALIBRATION *tpc = &tp_calib_data;

			global_inX = x = S12AD.ADDR1;
			global_inY = y = S12AD.ADDR0;
#ifdef PRNT_E1
	printf("\nTouch position X=%d,Y=%d",x,y);
#endif			
			x = ((x - tpc->x_min) * 320)/(tpc->x_max - tpc->x_min);
			if (x <= 0) x = 1;
			if (x >= 320) x = 319;
			y = ((tpc->y_max - y) * 240)/(tpc->y_max - tpc->y_min);
			if (y <= 0) y = 1;
			if (y >= 240) y = 239;
			if (global_touch == 0) ev_add(EV_PENDOWN, x, y);
                        pen_down_x = x;
                        pen_down_y = y;
			global_touch = 1;
		}
		PORTC.DR.BYTE = T_idle_DR;		//AChOԂɖ߂
		PORTC.DDR.BYTE = T_idle_DDR;		
		t_state = 0;
		break;
	default:
		t_state = 0;
		break;
	}
}

//^b`plp̓d
void set_idle(void){
	PORTC.DR.BYTE = T_idle_DR;
	PORTC.DDR.BYTE = T_idle_DDR;		
}

void chk_touch(void){
	PORTC.DR.BYTE = T_sense_DR;
	PORTC.DDR.BYTE = T_sense_DDR;
}

unsigned char chk_sense(void){
	return PORT4.PORT.BIT.B0;
}

void set_adc_x(void){
	PORTC.DR.BYTE = T_posX_DR;
	PORTC.DDR.BYTE = T_posX_DDR;
}

unsigned short get_adc_x(void){
	S12AD.ADANS.WORD = 2;		//AN1 enable
	S12AD.ADCSR.BIT.ADST = 1;	//strat
	while(!S12AD.ADCSR.BIT.ADST);
	return (S12AD.ADDR1);
}

void set_adc_y(void){
	PORTC.DR.BYTE = T_posY_DR;
	PORTC.DDR.BYTE = T_posY_DDR;
}

unsigned short get_adc_y(void){
	S12AD.ADANS.WORD = 1;		//AN0 enable
	S12AD.ADCSR.BIT.ADST = 1;	//strat
	while(!S12AD.ADCSR.BIT.ADST);
	return (S12AD.ADDR0);
}

//^b`pl̃Lu[V
void cal_touch(TP_CALIBRATION *tpc)
{
    int i,x,y;

        memset(tpc, 0, sizeof(TP_CALIBRATION));
	GCON_clear();
	fcrcl(8,8,12,12,C_BLACK,0);
        GCON_locate(10, 8);
        t_printf("4Ƀ^b`Ă");
	GCON_locate(2, 2);
	t_printf("TOUCH");

	x = 0;y = 0;
	for(i=0;i<16;i++){						//16񕪂̃f[^擾
		while(global_touch==0);
		x += global_inX; 
		y += global_inY;
	}
	if(x>20000) tpc->x_max += x/16;				//㉺AEtłΉ
	else tpc->x_min += x/16;
	if(y>20000) tpc->y_max += y/16;
	else tpc->y_min += y/16;
	while(global_touch==1);					//y܂ő҂

	GCON_clear();
	fcrcl(308,8,312,12,C_BLACK,0);
        GCON_locate(10, 8);
        t_printf("4Ƀ^b`Ă");
	GCON_locate(38, 2);
	t_printf("TOUCH");

	x = 0;y = 0;
	for(i=0;i<16;i++){
		while(global_touch==0);
		x += global_inX; 
		y += global_inY;
	}
	if(x>20000) tpc->x_max += x/16;
	else tpc->x_min += x/16;
	if(y>20000) tpc->y_max += y/16;
	else tpc->y_min += y/16;
	while(global_touch==1);

	GCON_clear();
	fcrcl(8,228,12,232,C_BLACK,0);
        GCON_locate(10, 8);
        t_printf("4Ƀ^b`Ă");
	GCON_locate(2, 16);
	t_printf("TOUCH");

	x = 0;y = 0;
	for(i=0;i<16;i++){
		while(global_touch==0);
		x += global_inX; 
		y += global_inY;
	}
	if(x>20000) tpc->x_max += x/16;
	else tpc->x_min += x/16;
	if(y>20000) tpc->y_max += y/16;
	else tpc->y_min += y/16;
	while(global_touch==1);

	GCON_clear();
	fcrcl(308,228,312,232,C_BLACK,0);
        GCON_locate(10, 8);
        t_printf("4Ƀ^b`Ă");
	GCON_locate(38, 16);
	t_printf("TOUCH");

	x = 0;y = 0;
	for(i=0;i<16;i++){
		while(global_touch==0);
		x += global_inX; 
		y += global_inY;
	}
	if(x>20000) tpc->x_max += x/16;
	else tpc->x_min += x/16;
	if(y>20000) tpc->y_max += y/16;
	else tpc->y_min += y/16;
	while(global_touch==1);
	
	tpc->x_max += (tpc->x_max - tpc->x_min)/300*10;		//300hbg320hbgւ̕ϊ
	tpc->x_min -= (tpc->x_max - tpc->x_min)/300*10;
	tpc->y_max += (tpc->y_max - tpc->y_min)/220*10;		//220hbg240hbgւ̕ϊ
	tpc->y_min -= (tpc->y_max - tpc->y_min)/220*10;
	tpc->x_max /= 2;
	tpc->x_min /= 2;
	tpc->y_max /= 2;
	tpc->y_min /= 2;

	GCON_clear();
        GCON_locate(12, 7);
        df_write(tpc);
        t_printf("Lu[V");
}
